use Test::More tests => 23 ;
use Stanford::Certificate::Config ;
use Data::Dumper ;

my $config ;

# Load with no config file and do NOT defer loading
Stanford::Certificate::Config::destroy() ;
eval {
  $config = Stanford::Certificate::Config::instance(
    'config_yaml_file'       => '/dev/null/bogus',
    ) ;
} ;
ok ($@ =~ m{cannot find the config YAML file}) ;

# Load with a non-existent config file (should die).
eval
{
  $config = Stanford::Certificate::Config::instance('config_yaml_file' => '/dev/null/bogus') ;
} ;
ok ($@ =~ m{cannot find the config YAML file}) ;

# Load with a real file, but one that is not in YAML format
my $my_not_yaml_file = './t/config_not_yaml.txt' ;
ok (-e $my_not_yaml_file) ;
eval
{
  $config = Stanford::Certificate::Config::instance('config_yaml_file' => $my_not_yaml_file) ;
} ;
ok ($@ =~ m{YAML::Tiny failed}) ;

# Load the REAL config YAML file.
my $config_yaml_file = './t/config.yml' ;
Stanford::Certificate::Config::destroy() ;
$config = Stanford::Certificate::Config::instance('config_yaml_file' => $config_yaml_file) ;
ok ($config->validate_hostname_access_via_netdb() == 1) ;
ok ($config->incommon_cm_credentials_yaml_file()  eq './cm_credentials.yml') ;
ok ($config->db_connect_yaml_file()               eq './sslcert_db_testing.yml') ;
ok ($config->to_email_override()                  eq 'abc123@stanford.edu') ;
ok ($config->cc_warning_emails()                  eq 'bogusbogusbogus@stanford.net') ;

my @approvers = @{ $config->incommon_approvers() } ;
ok (scalar(@approvers) == 2) ;
ok ($approvers[0] eq 'adamhl@stanford.edu') ;
ok ($approvers[1] eq 'zzz@sss.yyy') ;

# Load again (is a singleton so should be a no-op)
$config = Stanford::Certificate::Config::instance() ;

# Load the
# $config->config_yaml_file($my_not_yaml_file) ;

# Convert the config to string
my $valid_string ;
$valid_string = <<'EOC';
config_yaml_file:                  ./t/config.yml
incommon_cm_credentials_yaml_file: ./cm_credentials.yml
use_prod_certmanager:              NO
db_connect_yaml_file:              ./sslcert_db_testing.yml
incommon_approvers:                adamhl@stanford.edu, zzz@sss.yyy
from_email_address:                xyzqqqzzzz@stanford.edu
to_email_override:                 abc123@stanford.edu
enable_validation:                 YES
cc_warning_emails:                 bogusbogusbogus@stanford.net
EOC
ok ($config->to_s() eq $valid_string) ;

# Set approvers array to be empty.
$config->incommon_approvers([]) ;

$valid_string = <<'EOC';
config_yaml_file:                  ./t/config.yml
incommon_cm_credentials_yaml_file: ./cm_credentials.yml
use_prod_certmanager:              NO
db_connect_yaml_file:              ./sslcert_db_testing.yml
incommon_approvers:                NONE
from_email_address:                xyzqqqzzzz@stanford.edu
to_email_override:                 abc123@stanford.edu
enable_validation:                 YES
cc_warning_emails:                 bogusbogusbogus@stanford.net
EOC

ok ($config->to_s() eq $valid_string) ;

# Set email override to whitespace
$config->to_email_override('  ') ;

$valid_string = <<'EOC';
config_yaml_file:                  ./t/config.yml
incommon_cm_credentials_yaml_file: ./cm_credentials.yml
use_prod_certmanager:              NO
db_connect_yaml_file:              ./sslcert_db_testing.yml
incommon_approvers:                NONE
from_email_address:                xyzqqqzzzz@stanford.edu
to_email_override:                 <NONE>
enable_validation:                 YES
cc_warning_emails:                 bogusbogusbogus@stanford.net
EOC
ok ($config->to_s() eq $valid_string) ;

# Set from_email_address to something
$config->from_email_address('nobody2@stanford.edu') ;
$valid_string = <<'EOC';
config_yaml_file:                  ./t/config.yml
incommon_cm_credentials_yaml_file: ./cm_credentials.yml
use_prod_certmanager:              NO
db_connect_yaml_file:              ./sslcert_db_testing.yml
incommon_approvers:                NONE
from_email_address:                nobody2@stanford.edu
to_email_override:                 <NONE>
enable_validation:                 YES
cc_warning_emails:                 bogusbogusbogus@stanford.net
EOC
ok ($config->to_s() eq $valid_string) ;


# Load from config yml where use_prod_certmanager is missing (note that
# cc_warning_emails is also missing).
Stanford::Certificate::Config::destroy() ;
$config = Stanford::Certificate::Config::instance('config_yaml_file' => './t/config_missing1.yml') ;
ok (!defined($config->cc_warning_emails())) ;

# Load from config yml where use_prod_certmanager is YES
Stanford::Certificate::Config::destroy() ;
$config = Stanford::Certificate::Config::instance('config_yaml_file' => './t/config_prodyes.yml') ;
ok ($config->use_prod_certmanager()) ;

Stanford::Certificate::Config::destroy() ;
$config = Stanford::Certificate::Config::instance('config_yaml_file' => './t/config_missing2.yml') ;
ok ($config->to_email_override() eq q{}) ;

# If the netdb directive is missing, it should default to TRUE.
Stanford::Certificate::Config::destroy() ;
$config = Stanford::Certificate::Config::instance('config_yaml_file' => './t/config_missing_netdb.yml') ;
ok ($config->validate_hostname_access_via_netdb()) ;

# Load a config where netdb directive is NO
Stanford::Certificate::Config::destroy() ;
$config = Stanford::Certificate::Config::instance('config_yaml_file' => './t/config_netdb_no.yml') ;
ok (!$config->validate_hostname_access_via_netdb()) ;

# Load a config where netdb directive uses something bad
Stanford::Certificate::Config::destroy() ;
eval
{
  $config = Stanford::Certificate::Config::instance('config_yaml_file' => './t/config_netdb_bad.yml') ;
} ;
ok ($@ =~ m{validate_hostname_access_via_netdb config parameter not understood}) ;

# If the use_prod site directive is missing, it should default to FALSE.
Stanford::Certificate::Config::destroy() ;
$config = Stanford::Certificate::Config::instance('config_yaml_file' => './t/config_missing_prod.yml') ;
ok (!$config->use_prod_certmanager()) ;
