use Test::More ;

use Stanford::Orange::Util qw/ trim slurp spew tempfile / ;
use Stanford::Certificate::Misc qw/ mysql_time mysql_datetime_to_datecalc dedupe_array / ;
use Stanford::Certificate::Notification ;
use Stanford::Certificate::Config ;
use Data::Dumper ;

# Set the DEB_PACKAGE_BUILD environment variable.
require 't/run_non_build_tests.pl' ;

my $number_of_tests_run ;

# Test the username/password function
my $credentials_file = 't/cm_credentials_testing.yml' ;

my ($username, $password) ;
($username, $password) = Stanford::Certificate::Misc::get_username_password($credentials_file) ;
ok ($username eq 'adamhl') ;
ok ($password =~ m{COOKIE}) ;

## What if we don't give a credentials file (should use config default).
#($username, $password) = Stanford::Certificate::Misc::get_username_password() ;
#ok ($username eq 'adamhl') ;
#ok ($password =~ m{COOKIE}) ;

# What if we give a non-existent credentials file?
eval
{
  Stanford::Certificate::Misc::get_username_password('/dev/null/bogus') ;
} ;

ok ($@ =~ m{could not find credentials file}) ;



# Test mysql_time
ok (mysql_time(1359906220) eq '2013-02-03 15:43:40') ;

# Test converting MySQL date-time strings into Date::Calc objects.
my ($dc, $mysql_dt) ;

$mysql_dt = '2012-10-29 14:50:06' ;
$dc = mysql_datetime_to_datecalc($mysql_dt) ;
ok ($dc->string(1) eq '29-Oct-2012 14:50:06') ;

$mysql_dt = '2022-04-04 00:59:59' ;
$dc = mysql_datetime_to_datecalc($mysql_dt) ;
ok ($dc->string(1) eq '04-Apr-2022 00:59:59') ;

$mysql_dt = '2022-004 00:59:59' ;
eval
{
  $dc = mysql_datetime_to_datecalc($mysql_dt) ;
} ;

ok ($@ =~ m{could not parse MySQL date string}) ;

# Make sure Time::Zone is working properly (if not, then mysql_time_local
# will fail in strange ways).
use Time::Zone;
ok (defined(tz_offset('GMT')) && (tz_offset('GMT') == 0)) ;

my $tz_string = 'America/Los_Angeles' ;
# Convert a MySQL GMT time string to local time string.
my $mysql_gmt ;

# This is during DST:
$mysql_gmt = '2012-10-08 17:22:22' ;

ok (Stanford::Certificate::Misc::mysql_time_local($mysql_gmt, $tz_string)
     eq '2012-10-08 10:22:22') ;

# Do a bad date
eval
{
  Stanford::Certificate::Misc::mysql_time_local('2012-10-08 17:22', $tz_string) ;
} ;
ok ($@ =~ m{could not parse MySQL time string}) ;



# This is NOT during DST:
$mysql_gmt = '2012-12-29 03:00:23' ;
ok (Stanford::Certificate::Misc::mysql_time_local($mysql_gmt, $tz_string)
     eq '2012-12-28 19:00:23') ;

ok (Stanford::Certificate::Misc::mysql_to_friendly_date('2012-12-29 03:00:23')
     eq '29-Dec-2012') ;

ok (Stanford::Certificate::Misc::mysql_to_friendly_date('2018-01-01 23:59:59')
     eq '1-Jan-2018') ;

eval
{
  Stanford::Certificate::Misc::mysql_to_friendly_date('2018--01 23:59:59') ;
} ;
ok ($@ =~ m{could not parse MySQL date string}) ;


# Test Stanford::Certificate::Notification::days_from_now

# Test Stanford::Certificate::Notification::days_from_now
my $dc_future ;

## ######## ######## ######## ######## ######## ######## ######
my $compare_dates_fref = sub
{
  my ($days_from_now, $mysql_date_string) = @_ ;
  my $dc_days_from_now = Stanford::Certificate::Notification::days_from_now($dc, $days_from_now) ;
  my $dc_old_way = mysql_datetime_to_datecalc($mysql_date_string) ;
  return ($dc_days_from_now == $dc_old_way) ;
} ;
## ######## ######## ######## ######## ######## ######## ######

# Today is '2022-04-04 00:59:59'
ok ($compare_dates_fref->(1,  '2022-04-05 00:59:59')) ;
ok ($compare_dates_fref->(0,  '2022-04-04 00:59:59')) ;
ok ($compare_dates_fref->(7,  '2022-04-11 00:59:59')) ;
ok ($compare_dates_fref->(30, '2022-05-04 00:59:59')) ;
ok ($compare_dates_fref->(61, '2022-06-04 00:59:59')) ;

# Test de-duping
my @dedupes ;
ok (scalar(dedupe_array([])) == 0) ;

@dedupes = dedupe_array(['a']) ;
ok (scalar(@dedupes) == 1) ;
ok ($dedupes[0] eq 'a') ;

@dedupes = dedupe_array(['a', 'a']) ;
ok (scalar(@dedupes) == 1) ;
ok ($dedupes[0] eq 'a') ;

@dedupes = dedupe_array(['a', 'b', 'a']) ;
ok (scalar(@dedupes) == 2) ;
ok (($dedupes[0] eq 'a') && ($dedupes[1] eq 'b')) ;

@dedupes = dedupe_array(['a', 'a', 'b']) ;
ok (scalar(@dedupes) == 2) ;
ok (($dedupes[0] eq 'a') && ($dedupes[1] eq 'b')) ;

# Get the config
$config = Stanford::Certificate::Config::instance('config_yaml_file' => './t/config.yml') ;

SKIP:
{
  if (defined($ENV{'DEB_PACKAGE_BUILD'}) && ($ENV{'DEB_PACKAGE_BUILD'} eq '1'))
  {
    skip "skipping cert types check", 2 ;
  }

  # cert_types
  my @cert_types = Stanford::Certificate::Misc::cert_types($config->incommon_cm_credentials_yaml_file(), 1) ;

  ok (scalar(@cert_types) > 3) ;
  ok ($cert_types[0] =~ m{InCommon}) ;
}


# Test downloading a live certificate (but not when building!)
SKIP:
{
  if (defined($ENV{'DEB_PACKAGE_BUILD'}) && ($ENV{'DEB_PACKAGE_BUILD'} eq '1'))
  {
    skip "skipping live download test", 2 ;
  }
  my $cn = 'weblogin.stanford.edu' ;
  my $pkobj = Stanford::Certificate::Misc::download_live_certificate($cn) ;
  ok ($pkobj) ;
  ok ($pkobj->summary() =~ m{weblogin.stanford.edu}) ;
}


done_testing( $number_of_tests_run ) ;
